"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var common_1 = require("../../common");
var groupProvider_1 = require("./groupProvider");
var RenderedContent_1 = require("./RenderedContent");
exports.RenderedContent = RenderedContent_1.RenderedContent;
var interfaces_1 = require("../../interfaces");
var v1_1 = tslib_1.__importDefault(require("uuid/v1"));
function renderGroupRenderer(sources, groupRenderer, singleRenderers) {
    var renderedSources = sources.sources.map(function (source) {
        if (source.source.data) {
            source.source.data.renderedContent = renderSingleRenderer(source, singleRenderers, true);
        }
        return source;
    });
    if (!groupRenderer) {
        return (react_1.default.createElement(react_1.default.Fragment, null, renderedSources.map(function (s) {
            if (s.source.data) {
                return s.source.data.renderedContent;
            }
            return null;
        })));
    }
    var Component = groupRenderer;
    var srcs = renderedSources.map(function (s) { return s.source; });
    return (react_1.default.createElement(groupProvider_1.GroupProvider, { sources: srcs },
        react_1.default.createElement(Component, { sources: srcs })));
}
function renderSingleRenderer(source, renderers, isGroup) {
    var renderedContent = source.source.data && source.source.data.renderedContent;
    if (!renderedContent) {
        return null;
    }
    var singleRenderer = renderers && renderers.find(function (r) { return r.sourceType.includes(source.source.type); });
    if (!singleRenderer) {
        return renderedContent;
    }
    var Component = singleRenderer.component;
    var s = isGroup
        ? tslib_1.__assign({}, source.source, { data: tslib_1.__assign({}, source.source.data, { renderedContent: renderedContent }) }) : source.source;
    return (react_1.default.createElement(Component, { source: s, renderedContent: renderedContent, isGroup: Boolean(isGroup), key: v1_1.default() }));
}
function render(sources, renderers) {
    var singleRenderers = renderers && renderers.single;
    var groupRenderers = renderers && renderers.group;
    return sources.map(function (source) {
        if (interfaces_1.isSourceWithOptions(source)) {
            return renderSingleRenderer(source, singleRenderers, false);
        }
        return renderGroupRenderer(source, groupRenderers, singleRenderers);
    });
}
var Content = function (_a) {
    var _b = _a.renderers, renderers = _b === void 0 ? {} : _b;
    var sources = common_1.useDCContext().sources;
    return react_1.default.createElement(react_1.default.Fragment, null, render(sources, renderers));
};
exports.Content = Content;
