"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var interfaces_1 = require("../interfaces");
var deepmerge_1 = tslib_1.__importDefault(require("deepmerge"));
function runExtractor(source, plugin, options) {
    var newData = plugin({
        source: source.source,
        options: options,
    });
    var newSource = tslib_1.__assign({}, source);
    newSource.source.data = tslib_1.__assign({}, source.source.data, newData);
    return newSource;
}
function runMutation(source, plugin, options) {
    var newContent = plugin({
        source: source.source,
        options: options,
    });
    var newSource = tslib_1.__assign({}, source);
    newSource.source.content = newContent;
    return newSource;
}
function runPlugin(source, pluginT, sourceGroupPluginOptions) {
    var pluginOptions = {};
    var plugin;
    if (interfaces_1.isPluginWithOptions(pluginT)) {
        pluginOptions = pluginT.options;
        plugin = pluginT.plugin;
    }
    else {
        plugin = pluginT;
    }
    if (!plugin.sourceTypes || !plugin.sourceTypes.includes(source.source.type)) {
        return source;
    }
    var sourcePlugins = source.pluginsOptions &&
        source.pluginsOptions.find(function (p) { return p.name === plugin.name; });
    var sourcePluginOptions = sourcePlugins && sourcePlugins.options;
    var options = deepmerge_1.default.all([
        pluginOptions || {},
        sourceGroupPluginOptions || {},
        sourcePluginOptions || {},
    ]);
    switch (plugin.type) {
        case interfaces_1.PluginType.EXTRACTOR:
            return runExtractor(source, plugin.fn, options);
        case interfaces_1.PluginType.MUTATION:
            return runMutation(source, plugin.fn, options);
        default:
            return source;
    }
}
function runPluginForGroup(sources, plugin) {
    var sourceGroupPluginOptions = sources.pluginsOptions &&
        sources.pluginsOptions.find(function (p) {
            if (interfaces_1.isPluginWithOptions(plugin)) {
                return p.name === plugin.plugin.name;
            }
            return p.name === plugin.name;
        });
    var processedSources = sources.sources.map(function (source) {
        return runPlugin(source, plugin, sourceGroupPluginOptions);
    });
    return tslib_1.__assign({}, sources, { sources: processedSources });
}
function runPlugins(sources, plugins) {
    if (!plugins || !plugins.length) {
        return sources;
    }
    var processedSources = [];
    plugins.map(function (plugin) {
        processedSources = sources.map(function (source) {
            if (interfaces_1.isSourceGroupWithOptions(source)) {
                return runPluginForGroup(source, plugin);
            }
            return runPlugin(source, plugin);
        });
    });
    return processedSources;
}
exports.runPlugins = runPlugins;
