"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var interfaces_1 = require("../interfaces");
var deepmerge_1 = tslib_1.__importDefault(require("deepmerge"));
var v1_1 = tslib_1.__importDefault(require("uuid/v1"));
function renderEngine(source, renderEngines, sourceGroupRenderEngineOptions) {
    var re = renderEngines.find(function (r) {
        if (interfaces_1.isRenderEngineWithOptions(r)) {
            return r.renderEngine.sourceTypes.includes(source.source.type);
        }
        return r.sourceTypes.includes(source.source.type);
    });
    if (!re) {
        return null;
    }
    var renderE;
    if (interfaces_1.isRenderEngineWithOptions(re)) {
        renderE = re;
    }
    else {
        renderE = {
            renderEngine: re,
            options: {},
        };
    }
    var Render = renderE.renderEngine.component;
    var renderEngineOptions = renderE.options;
    var sourceRenderEngineOptions = source.renderEngineOptions;
    var options = deepmerge_1.default.all([
        renderEngineOptions || {},
        sourceGroupRenderEngineOptions || {},
        sourceRenderEngineOptions || {},
    ]);
    var s = tslib_1.__assign({}, source.source, { data: tslib_1.__assign({}, source.source.data) });
    return react_1.default.createElement(Render, { source: s, options: options, key: v1_1.default() });
}
function renderGroup(sources, renderEngines) {
    return tslib_1.__assign({}, sources, { sources: sources.sources.map(function (source) {
            return renderSingle(source, renderEngines, sources.renderEngineOptions);
        }) });
}
function renderSingle(source, renderEngines, sourceGroupRenderEngineOptions) {
    if (source.source.data) {
        source.source.data.renderedContent = renderEngine(source, renderEngines, sourceGroupRenderEngineOptions);
    }
    return source;
}
function runRender(sources, renderEngines) {
    return sources.map(function (source) {
        if (interfaces_1.isSourceWithOptions(source)) {
            return renderSingle(source, renderEngines);
        }
        return renderGroup(source, renderEngines);
    });
}
exports.runRender = runRender;
