"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var interfaces_1 = require("../interfaces");
var plugins_1 = require("./plugins");
var render_1 = require("./render");
function serializeSource(source) {
    source.source.data = source.source.data || {};
    return source;
}
function serializeSourceGroup(group) {
    return tslib_1.__assign({}, group, { sources: group.sources.map(function (s) { return serializeSource(s); }) });
}
function serializeSources(sources) {
    return sources.map(function (source) {
        if (interfaces_1.isSourceGroupWithOptions(source)) {
            return serializeSourceGroup(source);
        }
        return serializeSource(source);
    });
}
function extractSource(source) {
    return source.source;
}
function extractSourceGroup(group) {
    return group.sources.map(function (s) { return extractSource(s); });
}
function extractSources(sources) {
    return sources.map(function (source) {
        if (interfaces_1.isSourceGroupWithOptions(source)) {
            return extractSourceGroup(source);
        }
        return extractSource(source);
    });
}
exports.extractSources = extractSources;
function runSystem(options) {
    var preProcessedSources = plugins_1.runPlugins(options.sources, options.plugins);
    preProcessedSources = serializeSources(options.sources);
    preProcessedSources = render_1.runRender(options.sources, options.renderEngines);
    return preProcessedSources;
}
exports.runSystem = runSystem;
